#coding: utf-8

import hashlib
import psutil
import time

DEFAULT_INTERVAL = 15

def emit(channel, info):
    channel.put(info)


def hash_disk_name(s):
    md5 = hashlib.md5()
    md5.update(s)
    return md5.hexdigest()


def collect(feed, config=None):
    disk_ids = {}
    previous_read = {}
    previous_written = {}

    if not config:
        config = {}

    interval = config.get('interval', DEFAULT_INTERVAL)

    # Чтобы избежать _внезапных_ пиков при первом цикле сборки скоростей,
    # заранее заполняем массив актуальным количество считанных и записанных байт.
    initial_disk_op = psutil.disk_io_counters(perdisk=True)
    for disk, counters in initial_disk_op.iteritems():
        previous_read[disk] = counters.read_bytes
        previous_written[disk] = counters.write_bytes

    while True:
        per_disk_io = psutil.disk_io_counters(perdisk=True)

        for disk_id, counters in per_disk_io.iteritems():
            # Избегаем постоянного вычисления хэша для известных партиций
            disk_hash = disk_ids.setdefault(disk_id,
                                            'disk_{0}'.format(hash_disk_name(disk_id)))
            r1 = previous_read[disk_id]
            r2 = counters.read_bytes
            bytes_read = int((r2 - r1) / interval)

            r1 = previous_written[disk_id]
            r2 = counters.write_bytes
            bytes_written = int((r2 - r1) / interval)

            emit(feed, ('{0}.io.read.bytes_per_sec'.format(disk_hash), bytes_read, 'gauge'))
            emit(feed, ('{0}.io.write.bytes_per_sec'.format(disk_hash), bytes_written, 'gauge'))

            previous_read[disk_id] = counters.read_bytes
            previous_written[disk_id] = counters.write_bytes

        time.sleep(interval)
